#!/usr/bin/env python
import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits.mplot3d import proj3d
import numpy as np
import matplotlib.pyplot as plt
import csv
import re
import sys
import os
import argparse as ap
import numpy.linalg as lg


parser=ap.ArgumentParser(description="Making an .ogv video from a set of trajectory.csv files")
parser.add_argument("-t","--trajectories",nargs='+',required=True, help="trajectory.csv files to parse data from")
parser.add_argument('-f',"--frames",type=int,default=400, help="number of frames in movie")
parser.add_argument('-o',"--output",default="carriers.ogv", help="Name of the output video")
args=parser.parse_args()

class carrier(object):
    numberofobjects= 0
    def __init__(self):
        carrier.numberofobjects+=1
        self.id=carrier.numberofobjects
        print("Charge {} has been added".format(self.id))
        self.t=[]
        self.x=[]
        self.y=[]
        self.z=[]

    def tmax(self):
        return self.t[-1]

    def append(self,x,y,z,t):
        self.x.append(x) 
        self.y.append(y) 
        self.z.append(z) 
        self.t.append(t) 

    def returnpos(self,timestep):
        return self.pos[np.argmin(np.absolute(self.pos[:,0]-timestep))]

    def returntraj(self,timestep):
        return self.pos[0:np.argmin(np.absolute(self.pos[:,0]-timestep))+1]
    
    def returntrajdim(self):
        return np.array([np.amin(self.pos[:,1]),np.amax(self.pos[:,1]),np.amin(self.pos[:,2]),np.amax(self.pos[:,2]),np.amin(self.pos[:,3]),np.amax(self.pos[:,3])])     
    
    def array(self):
        self.pos=np.array([self.t,self.x,self.y,self.z]).T   

    def colorcharge(self,color):
        self.color=color   
     
    def info(self):
        print("Carrier No. ",self.id)                
        print(self.array())


class carriercollection(object):
    
    def __init__(self,steps=False):
        self.listofcarriers=[]
        self.foldername="kmc_video"
        self.steps=steps
        self.colors=['#610B0B','#B40404','#DF7401','#AEB404','#3ADF00','#01A9DB','#0101DF','#A901DB','#BE3E3E','#3326BF','#13D2BC','#1AD213','#AFD213','#1AD213','#66EA41','#D641EA']
    def appendcharge(self):
        charge=carrier()
        self.listofcarriers.append(charge)
        

    def addtimestep(self,row):
        for i,carrier in enumerate(self.listofcarriers):
            carrier.append(float(row[3*i+1]),float(row[3*i+2]),float(row[3*i+3]),float(row[0]))
                            
    
    def tmin(self):
        tmin=1E25
        for carrier in self.listofcarriers:
            ttest=carrier.tmax()
            if ttest<tmin:
                tmin=ttest
        return tmin


    def boxdimension(self):
            box=[]
            for carrier in self.listofcarriers:
                box.append(carrier.returntrajdim())
            boxsizes=np.array(box)
            self.xinterval=1.2*np.array([np.min(boxsizes[:,0]),np.max(boxsizes[:,1])])
            self.yinterval=1.2*np.array([np.min(boxsizes[:,2]),np.max(boxsizes[:,3])])
            self.zinterval=1.2*np.array([np.min(boxsizes[:,4]),np.max(boxsizes[:,5])])
            print(self.xinterval,self.yinterval,self.zinterval)
    def mergeonto(self,other):
        self.listofcarriers=self.listofcarriers+other.listofcarriers

    def assigncolors(self):
        i=0
        while len(self.listofcarriers)>len(self.colors):
            self.colors=self.colors+self.colors
        for carrier in self.listofcarriers:
            carrier.colorcharge(self.colors[i])
            i+=1
            
        
    def readinfile(self, infile):   
        with open (infile, "rb") as csvfile:
            print("Opening file {}".format(infile))
            reader = csv.reader(csvfile, dialect="excel-tab")
            for row in reader:
                if  reader.line_num>self.steps and self.steps!=False:
                    break
                if "time" in row[0]:
                    if "charges" in ''.join(row):
                        print(''.join(row))
                        m=re.search('\( ([0-9].?) charges \)',''.join(row))
                        noofcharges=int(m.group(1))
                    else:
                        noofcharges=len(row)/3

                    for i in range(noofcharges):
                        self.appendcharge()  

                else:
                        self.addtimestep(row)
            for carrier in self.listofcarriers:
                carrier.array()

            print("Found {} charges in file.".format(noofcharges))
    

    def plottimestep(self,time,step):
        #markers=r'$\ddot\smile$'
        markers=r'$\ddot\frown$'
        fig = plt.figure()
        ax = fig.gca(projection='3d')
        timetext="t={:3e} s".format(time)
        print(timetext)
        ax.set_title(timetext,loc='center')
        ax.set_xlabel('x in [nm]')
        ax.set_ylabel('y in [nm]')
        ax.set_zlabel('z in [nm]')
        ax.set_xlim(self.xinterval[0],self.xinterval[1])
        ax.set_ylim(self.yinterval[0],self.yinterval[1])
        ax.set_zlim(self.zinterval[0],self.zinterval[1])
        for carrier in self.listofcarriers:
            posarray=carrier.returntraj(time)
            #print(posarray)
            ax.plot(posarray[:,1], posarray[:,2], posarray[:,3],c=carrier.color) 
            ax.scatter(posarray[-1,1], posarray[-1,2], posarray[-1,3],s=250,marker="o",c='#FFFF00')     
            ax.scatter(posarray[-1,1], posarray[-1,2], posarray[-1,3],s=150,marker=markers,c='#FFFF00')  
            ax.scatter(posarray[0,1], posarray[0,2], posarray[0,3],s=400,marker="x",c=carrier.color)
        plt.draw()
        filename = 'frame_{0:04d}.png'.format(step)
        plt.savefig(filename,dpi=100,bbox_inches="tight",pad_inches=0.0)
        plt.close() 
        return 0

    def createfolder(self,i):

        if os.path.isdir(self.foldername+str(i)) and (os.path.isdir(self.foldername+str(i-1)) or (os.path.isdir(self.foldername) and i==1)):
            os.rename(self.foldername+str(i),self.foldername+str(i+1))
            print("backing up folder {} to {}".format(self.foldername+str(i),self.foldername+str(i+1)))
        if os.path.isdir(self.foldername) and i==0:
            os.rename(self.foldername,self.foldername+str(i+1))
            print("backing up folder {} to {}".format(self.foldername,self.foldername+str(i+1)))
        if i==0:
            os.mkdir(self.foldername)
            print("creating folder {}".format(self.foldername))
        elif i>0:
            self.createfolder(i-1)
        else:
            print("Something about the foldernames is not right. Exiting....")
            sys.exit()

    def makemovie(self,filename,length):
        framerate=25
        self.assigncolors()
        self.boxdimension()
        os.chdir(self.foldername)
        tmin=self.tmin()
        print(tmin)
        steps=range(int(length*framerate)+1)
        for step in steps:
            print("evaluating frame {} of {}.".format(step,length*framerate))
            time=step*tmin/(length*framerate)
            self.plottimestep(time,step)
        command='ffmpeg -i frame_%04d.png {}'.format(filename)
        os.system(command)
        


step=args.frames
trajectories=carriercollection(step)

for csvfile in args.trajectories:
    print(csvfile)
    temptraj=carriercollection(step)
    temptraj.readinfile(csvfile)
    trajectories.mergeonto(temptraj)
    
trajectories.createfolder(20)
length=args.frames/25
trajectories.makemovie(args.output,length)
    




#!/usr/bin/python

import lxml.etree as lxml
import argparse as ap
parser=ap.ArgumentParser(description="")
parser.add_argument("-j","--jobfile",required=True, help="jobfile to parse data from")
args=parser.parse_args()


xmlparser=lxml.XMLParser(remove_comments=True)
tree = lxml.parse(args.jobfile,xmlparser)
root = tree.getroot()
for entry in root.iter('job'): 
    output=entry.find("output")
    regions=output.find("regions")
    region=regions.find("region")
    estat=region.find("E_static").text
    output.find("E_tot").text=estat

print(lxml.tostring(root, pretty_print=True))



#
# Copyright 2009-2020 The VOTCA Development Team (http://www.votca.org)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Module to edit inplace the content of xml files."""

__all__ = ["edit_calculator", "add_section"]

import io
import xml.etree.ElementTree as ET
from pathlib import Path
from typing import List, Tuple


def get_tree_and_root(file_name: Path) -> Tuple[ET.Element, ET.Element]:
    """Get the tree and root of a calculator input."""
    if not file_name.exists():
        raise RuntimeError(f"There is no file: {file_name}")
    # Read XML file
    tree = ET.parse(file_name)
    root = tree.getroot()

    return tree, root


def edit_calculator(calculator: str, option: str, value: str, folder: str = "OPTIONFILES") -> None:
    """Edit ``value`` from ``option`` of calculator."""
    file_name = Path(folder) / Path(f"{calculator}.xml")
    tree, root = get_tree_and_root(file_name)
    edit_option(list(root)[0], option, value)
    # Finally overwrite the file
    tree.write(file_name)
    msg = f"The option '{option}' on file '{file_name.as_posix()}' has been set to '{value}'"
    print(msg)


def edit_option(elem: ET.Element, option: str, value: str) -> None:
    """Change value of ``option`` in ``elem``."""
    children = find_nodes(elem, option)
    if not children:
        raise RuntimeError(f"There is not {option} in {elem}")
    else:
        for child in children:
            child.text = value
            if "default" in child.attrib:
                del child.attrib["default"]


def find_nodes(elem: ET.Element, option: str) -> List[ET.Element]:
    """Find ``option`` in ``elem`` and in all its children."""
    sections = option.split('.')
    if len(sections) == 1:
        acc = []
        find_nodes_recursively(elem, option, acc)
        return acc
    else:
        return find_section(elem, sections)


def find_section(elem: ET.Element, sections: List[str]) -> List[ET.Element]:
    """Search recursively for '.' separated section in elem."""
    for name in sections:
        elem = elem.find(name)
        if elem is None:
            break

    return [elem]


def find_nodes_recursively(elem: ET.Element, option: str, acc: List[ET.Element]) -> None:
    """Find ``option`` in ``elem`` and in all its children."""
    child = elem.find(option)
    if child is not None:
        acc.append(child)
    # If elem has children
    if list(elem):
        for c in list(elem):
            find_nodes_recursively(c, option, acc)


def add_section(calculator: str, xml_string: str, folder: str = "OPTIONFILES") -> None:
    """Add a new element to the ``calculator`` input."""
    file_name = Path(folder) / Path(f"{calculator}.xml")
    tree, root = get_tree_and_root(file_name)
    new_elem = ET.parse(io.StringIO(xml_string))
    calculator = next(iter(root))
    calculator.append(new_elem.getroot())
    tree.write(file_name)



import h5py
import sys
import numpy as np
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("-f", "--orbfile",required=True, help="checkpoint HDF5 file",)
parser.add_argument("-o", "--outputfile",default="spectrum.dat", help="output file for spectra")
args = parser.parse_args()

def getEnergies(orb):
    a=orb['region_0']['orbitals']['BSE_singlet']['eigenvalues'].value
    a.flatten()
    return a.flatten()

def trans_sort(index):
    return int(index[3:])

def getOscillators(orb):
    energies=getEnergies(orb)
    transdip=[]
    for k in sorted(orb['region_0']['orbitals']['transition_dipoles'].keys(),key=trans_sort):
        transdip.append(np.array(orb['region_0']['orbitals']['transition_dipoles'][k].value)) 
    d2=[]
    for b in transdip:
        d2.append(np.sum(b**2))    
    d2=np.array(d2)
    oscs=2/3.0*energies*d2
    return oscs

orb=h5py.File(args.orbfile,'r')
e=getEnergies(orb)*27.2114
osc=getOscillators(orb)
output=np.array([e,osc])
np.savetxt(args.outputfile,output.T,header="Omega[eV],f")
print("Wrote spectrum to {}.".format(args.outputfile))
